package com.framsticks.framclipse.framScript;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.framsticks.framclipse.framScript.impl.FunctionImpl;
import com.framsticks.framclipse.script.model.Argument;
import com.framsticks.framclipse.script.model.Element;

public class ProposableFunctionImpl extends FunctionImpl implements Proposable {

	private final String display;
	private final String proposal;
	private final Map<String, String> description = new LinkedHashMap<String, String>();

	public ProposableFunctionImpl(Element element) {
		assert element.isFunction();
		name = element.getName();
		display = name + element.argumentsString(true) + " : " + element.returnType();
		proposal = name + element.argumentsString(false) + ("void".equals(element.returnType()) ? ";" : "");
		createDescription(element);
	}

	private void createDescription(Element element) {
		description.put("Description", element.getDescription());
		description.put("Returns", element.returnType());
		description.put("Arguments", createArguments(element.getArguments()));
	}

	private String createArguments(List<Argument> args) {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < args.size(); i++) {
			Argument a = args.get(i);
			builder.append(a.getName(i) + " : " + a.getType() + "\n");
		}
		return builder.toString();
	}

	@Override
	public String display() {
		return display;
	}

	@Override
	public String proposal() {
		return proposal;
	}

	@Override
	public Map<String, String> description() {
		return description;
	}

}
