package com.framsticks.framclipse.script;

import java.util.ArrayDeque;
import java.util.Queue;

import org.eclipse.emf.ecore.EObject;

import com.framsticks.framclipse.framScript.ArrayElementExpression;
import com.framsticks.framclipse.framScript.Invocation;
import com.framsticks.framclipse.framScript.QualifiedExpression;
import com.framsticks.framclipse.framScript.VariableRef;
import com.framsticks.framclipse.script.model.Element;
import com.framsticks.framclipse.script.model.Type;
import com.google.inject.Inject;

public class ExpressionTraverser {

	@Inject
	private ConstantProvider provider;

	public Type getLastType(QualifiedExpression expr) {
		String ext = getExtension(expr);
		Queue<String> fragments = getExpressionFragments(expr);
		Type type = provider.getType(fragments.poll(), ext);

		while (!fragments.isEmpty() && type != null) {
			Element element = type.getElement(fragments.poll());
			type = element != null ? provider.getType(element.getType(), ext) : null;
		}
		return type;
	}

	public Element getLastElement(QualifiedExpression expression) {
		if (expression.eContainer() instanceof QualifiedExpression) {
			QualifiedExpression container = (QualifiedExpression) expression.eContainer();
			if (expression == container.getChild()) { // last part of complex expression
				String last = getFragmentName(expression);
				Type type = getLastType(container);
				return type != null ? type.getElement(last) : null;
			} else {
				Element element = getLastElement(container);
				if (element != null) {
					return element;
				}
			}
		}
		return null;
	}

	public Element getLastElementFromContainer(QualifiedExpression container) {
		String ext = getExtension(container);
		Queue<String> fragments = getExpressionFragments(container);
		Type type = provider.getType(fragments.poll(), ext);
		Element element = type != null ? type.getElement(fragments.poll()) : null;

		while (!fragments.isEmpty() && element != null) {
			type = element != null ? provider.getType(element.getType(), ext) : null;
			element = type.getElement(fragments.poll());
		}
		return element;
	}

	public Queue<String> getExpressionFragments(QualifiedExpression expr) {
		return getExpressionFragments(expr, new ArrayDeque<String>());
	}

	protected Queue<String> getExpressionFragments(QualifiedExpression expr, Queue<String> fragments) {
		QualifiedExpression parent = expr.getParent();
		if (expr != null && parent != null) {
			EObject eContainer = expr.eContainer();
			if (eContainer instanceof QualifiedExpression) {
				getExpressionFragments((QualifiedExpression) eContainer, fragments);
			}
			if (parent instanceof ArrayElementExpression) {
				ArrayElementExpression ae = (ArrayElementExpression) parent;
				if (ae.getArray() != null) {
					fragments.add(getFragmentName(ae.getArray()));
					for (int i = 0; i < ae.getIndexes().size(); i++) {
						fragments.add("get");
					}
				}
			}
			String name = getFragmentName(parent);
			if (name != null) {
				fragments.add(name);
			}
		}
		return fragments;
	}

	public String getFragmentName(QualifiedExpression expr) {
		if (expr instanceof VariableRef) {
			return ((VariableRef) expr).getVar().getName();
		} else if (expr instanceof Invocation) {
			return ((Invocation) expr).getFunction().getName();
		}
		return null; // FIXME azer staf
	}
	
	private String getExtension(EObject object) {
		return object.eResource().getURI().fileExtension();
	}

}
